
include<dynamixel.scad>;

screw_d = 3;
femur_t = 2;

module femur_screw_pos(tibiaOffset) {
    translate([ sqrt(2) * XL320_dim[2] / 2 + femur_t + screw_d / 2, 0, 0 ])
        children();
    translate(tibiaOffset)
        translate([ -sqrt(2) * XL320_dim[2] / 2 - femur_t - screw_d / 2, 0, 0 ])
            children();
}

module scaled_tube(x, y, t) {
    difference() {
        scale([ x, y, t ]) cylinder(d = 1, h = 1);
        translate([ 0, 0, -0.5 ]) scale([ x - t * 2, y - t * 2, t + 1 ])
            cylinder(d = 1, h = 1);
    }
}

module femur_front_side(tibiaOffset) difference() {
    $fn = 32;
    d = XL320_horn_d;

    s = XL320_dim[1] / 2 + XL320_horn_h;

    translate([ 0, -s, 0 ]) rotate([ 90, 0, 0 ]) {
        translate([ tibiaOffset[0] / 2, 0, 0 ]) {
            scaled_tube(tibiaOffset[0] + d, d, femur_t);
            scaled_tube(tibiaOffset[0] - XL320_dim[2] * sqrt(2), d, femur_t);
        }
        femur_screw_pos(tibiaOffset)
            tube(screw_d, screw_d + femur_t * 2, femur_t);

        for (p = [ [ 0, 0, 0 ], tibiaOffset ]) translate(p) {
                XL320_horn_adapter();
            }
    }

    femur_screw_pos(tibiaOffset) rotate([ 90, 0, 0 ])
        cylinder(d = screw_d, h = 100, center = true);

    for (p = [ [ 0, 0, 0 ], tibiaOffset ])
        translate(p) rotate([ 90, 0, 0 ]) XL320_horn_hole_pos()
            cylinder(d = screw_d, h = 100, center = true);
}

module femur_rear_side(tibiaOffset) difference() {
    $fn = 32;
    d = screw_d + femur_t * 2;

    s = XL320_dim[1] / 2;
    translate([ 0, s, 0 ]) rotate([ -90, 0, 0 ]) {
        translate([ tibiaOffset[0] / 2, 0, 0 ]) {
            scaled_tube(tibiaOffset[0] + d, d * 2, femur_t);
            scaled_tube(tibiaOffset[0] - XL320_dim[2] * sqrt(2), d * 2,
                        femur_t);
        }
        femur_screw_pos(tibiaOffset)
            tube(screw_d, screw_d + femur_t * 2, femur_t);

        for (p = [ [ 0, 0, 0 ], tibiaOffset ])
            translate(p) cylinder(d = d, h = femur_t);
    }
    for (p = [ [ 0, 0, 0 ], tibiaOffset ])
        rotate([ 90, 0, 0 ]) translate(p)
            cylinder(d = screw_d, h = 100, center = true);

    femur_screw_pos(tibiaOffset) rotate([ 90, 0, 0 ])
        cylinder(d = screw_d, h = 100, center = true);
}

module femur_collision_a(tibiaOffset) {
    dim = [
        tibiaOffset[0] + XL320_horn_d + femur_t, femur_t, XL320_horn_d + femur_t
    ];
    translate([ 0, -XL320_dim[1] / 2 - dim[1] / 2 - XL320_horn_h, 0 ])
        translate(tibiaOffset / 2) cube(dim, center = true);
}

module femur_collision_b(tibiaOffset) {
    dim = [
        tibiaOffset[0] + screw_d + 2 * femur_t, femur_t,
        screw_d * 2 + femur_t * 4
    ];
    translate([ 0, XL320_dim[1] / 2 + dim[1] / 2, 0 ])
        translate(tibiaOffset / 2) cube(dim, center = true);
}

module femur_collision_c(tibiaOffset) {
    dim = [
        screw_d + femur_t * 2, XL320_dim[1] + XL320_horn_h,
        screw_d + femur_t * 2
    ];
    translate([ 0, -XL320_horn_h / 2, 0 ]) hull() femur_screw_pos(tibiaOffset)
        cube(dim, center = true);
}

module femur(tibiaOffset) color("purple") {
    //    rotate([ 0, 180, 0 ]) XL320();
    //  XL320_horn();

    $fn = 32;

    femur_front_side(tibiaOffset);
    femur_rear_side(tibiaOffset);
    femur_screw_pos(tibiaOffset) {
        translate([ 0, XL320_dim[1] / 2, 0 ]) rotate([ 90, 0, 0 ])
            tube(screw_d, screw_d + femur_t * 2, XL320_dim[1] + XL320_horn_h);
    }

    // translate(tibiaOffset) {
    //  XL320();
    // XL320_horn();
    //}
}
